@@ -8,8 +8,9 @@
 #include <errno.h>
 #include <assert.h>
 #include <sys/mman.h>
-#include "fio.h"
-#include "os.h"
+
+#include "../fio.h"
+#include "../os.h"
 
 struct mmapio_data {
 	struct io_u *last_io_u;
@@ -88,7 +89,7 @@ static int fio_mmapio_init(struct thread_data *td)
 	return 0;
 }
 
-struct ioengine_ops ioengine = {
+static struct ioengine_ops ioengine = {
 	.name		= "mmap",
 	.version	= FIO_IOOPS_VERSION,
 	.init		= fio_mmapio_init,
@@ -98,3 +99,13 @@ struct ioengine_ops ioengine = {
 	.cleanup	= fio_mmapio_cleanup,
 	.flags		= FIO_SYNCIO | FIO_MMAPIO,
 };
+
+static void fio_init fio_mmapio_register(void)
+{
+	register_ioengine(&ioengine);
+}
+
+static void fio_exit fio_mmapio_unregister(void)
+{
+	unregister_ioengine(&ioengine);
+}
