@@ -93,15 +93,14 @@ static int create_files(struct thread_data *td)
 	struct fio_file *f;
 	int i, err, need_create;
 
+	for_each_file(td, f, i)
+		f->file_size = td->total_file_size / td->nr_files;
+
 	/*
 	 * unless specifically asked for overwrite, let normal io extend it
 	 */
-	if (!td->overwrite) {
-		for_each_file(td, f, i)
-			f->file_size = td->total_file_size / td->nr_files;
-
+	if (!td->overwrite)
 		return 0;
-	}
 
 	need_create = 0;
 	if (td->filetype == FIO_TYPE_FILE)
@@ -118,14 +117,19 @@ static int create_files(struct thread_data *td)
 	}
 
 	temp_stall_ts = 1;
-	fprintf(f_out, "%s: Laying out IO file(s) (%d x %LuMiB == %LuMiB)\n",
+	fprintf(f_out, "%s: Laying out IO file(s) (%u x %LuMiB == %LuMiB)\n",
 				td->name, td->nr_uniq_files,
 				(td->total_file_size >> 20) / td->nr_uniq_files,
 				td->total_file_size >> 20);
 
 	err = 0;
 	for_each_file(td, f, i) {
+		/*
+		 * Only unlink files that we created.
+		 */
+		f->unlink = 0;
 		if (file_ok(td, f)) {
+			f->unlink = td->unlink;
 			err = create_file(td, f);
 			if (err)
 				break;
@@ -150,9 +154,9 @@ static int file_size(struct thread_data *td, struct fio_file *f)
 
 		if (!f->file_size || f->file_size > f->real_file_size)
 			f->file_size = f->real_file_size;
-	}
+	} else
+		f->real_file_size = f->file_size;
 
-	f->file_size -= f->file_offset;
 	return 0;
 }
 
@@ -318,18 +322,18 @@ static int setup_file(struct thread_data *td, struct fio_file *f)
 
 	if (td->odirect)
 		flags |= OS_O_DIRECT;
+	if (td->sync_io)
+		flags |= O_SYNC;
 
 	if (td_write(td) || td_rw(td)) {
+		flags |= O_RDWR;
+
 		if (td->filetype == FIO_TYPE_FILE) {
 			if (!td->overwrite)
 				flags |= O_TRUNC;
 
 			flags |= O_CREAT;
 		}
-		if (td->sync_io)
-			flags |= O_SYNC;
-
-		flags |= O_RDWR;
 
 		f->fd = open(f->file_name, flags, 0600);
 	} else {
@@ -352,10 +356,34 @@ static int setup_file(struct thread_data *td, struct fio_file *f)
 	return 0;
 }
 
+int open_files(struct thread_data *td)
+{
+	struct fio_file *f;
+	int i, err = 0;
+
+	for_each_file(td, f, i) {
+		err = setup_file(td, f);
+		if (err)
+			break;
+	}
+
+	if (!err)
+		return 0;
+
+	for_each_file(td, f, i) {
+		if (f->fd != -1) {
+			close(f->fd);
+			f->fd = -1;
+		}
+	}
+
+	return err;
+}
+
 int setup_files(struct thread_data *td)
 {
 	struct fio_file *f;
-	int i, err;
+	int err, i;
 
 	/*
 	 * if ioengine defines a setup() method, it's responsible for
@@ -367,13 +395,7 @@ int setup_files(struct thread_data *td)
 	if (create_files(td))
 		return 1;
 
-	err = 0;
-	for_each_file(td, f, i) {
-		err = setup_file(td, f);
-		if (err)
-			break;
-	}
-
+	err = open_files(td);
 	if (err)
 		return err;
 
@@ -397,9 +419,18 @@ int setup_files(struct thread_data *td)
 	td->total_io_size = td->io_size * td->loops;
 
 	if (td->io_ops->flags & FIO_MMAPIO)
-		return setup_files_mmap(td);
+		err = setup_files_mmap(td);
 	else
-		return setup_files_plain(td);
+		err = setup_files_plain(td);
+
+	for_each_file(td, f, i) {
+		if (f->fd != -1) {
+			close(f->fd);
+			f->fd = -1;
+		}
+	}
+
+	return err;
 }
 
 void close_files(struct thread_data *td)
@@ -408,14 +439,13 @@ void close_files(struct thread_data *td)
 	int i;
 
 	for_each_file(td, f, i) {
-		if (f->fd != -1) {
-			if (td->unlink && td->filetype == FIO_TYPE_FILE &&
-			    td->filename) {
-				unlink(f->file_name);
-				td->filename = NULL;
-			}
+		if (!td->filename && f->unlink &&
+		    td->filetype == FIO_TYPE_FILE) {
+			unlink(f->file_name);
 			free(f->file_name);
 			f->file_name = NULL;
+		}
+		if (f->fd != -1) {
 			close(f->fd);
 			f->fd = -1;
 		}
@@ -425,6 +455,7 @@ void close_files(struct thread_data *td)
 		}
 	}
 
+	td->filename = NULL;
 	free(td->files);
 	td->files = NULL;
 	td->nr_files = 0;
