@@ -45,7 +45,6 @@ int groupid = 0;
 int thread_number = 0;
 int shm_id = 0;
 int temp_stall_ts;
-const char *fio_inst_prefix = _INST_PREFIX;
 
 static volatile int startup_sem;
 
@@ -106,7 +105,7 @@ static int check_min_rate(struct thread_data *td, struct timeval *now)
 
 		rate = (td->this_io_bytes[ddir] - td->rate_bytes) / spent;
 		if (rate < td->ratemin) {
-			fprintf(f_out, "%s: min rate %d not met, got %ldKiB/sec\n", td->name, td->ratemin, rate);
+			fprintf(f_out, "%s: min rate %u not met, got %luKiB/sec\n", td->name, td->ratemin, rate);
 			return 1;
 		}
 	}
@@ -243,9 +242,8 @@ static int fio_io_sync(struct thread_data *td, struct fio_file *f)
  * The main verify engine. Runs over the writes we previusly submitted,
  * reads the blocks back in, and checks the crc/md5 of the data.
  */
-void do_verify(struct thread_data *td)
+static void do_verify(struct thread_data *td)
 {
-	struct timeval t;
 	struct io_u *io_u, *v_io_u = NULL;
 	struct io_completion_data icd;
 	struct fio_file *f;
@@ -266,14 +264,15 @@ void do_verify(struct thread_data *td)
 		if (td->terminate)
 			break;
 
-		gettimeofday(&t, NULL);
-		if (runtime_exceeded(td, &t))
-			break;
-
 		io_u = __get_io_u(td);
 		if (!io_u)
 			break;
 
+		if (runtime_exceeded(td, &io_u->start_time)) {
+			put_io_u(td, io_u);
+			break;
+		}
+
 		if (get_next_verify(td, io_u)) {
 			put_io_u(td, io_u);
 			break;
@@ -314,6 +313,7 @@ void do_verify(struct thread_data *td)
 		v_io_u = td->io_ops->event(td, 0);
 		icd.nr = 1;
 		icd.error = 0;
+		fio_gettime(&icd.time, NULL);
 		io_completed(td, v_io_u, &icd);
 
 		if (icd.error) {
@@ -350,7 +350,7 @@ static void do_cpuio(struct thread_data *td)
 	int i = 0;
 
 	while (!td->terminate) {
-		gettimeofday(&e, NULL);
+		fio_gettime(&e, NULL);
 
 		if (runtime_exceeded(td, &e))
 			break;
@@ -371,7 +371,7 @@ static void do_cpuio(struct thread_data *td)
 static void do_io(struct thread_data *td)
 {
 	struct io_completion_data icd;
-	struct timeval s, e;
+	struct timeval s;
 	unsigned long usec;
 	struct fio_file *f;
 	int i, ret = 0;
@@ -433,26 +433,30 @@ static void do_io(struct thread_data *td)
 		 * of completions except the very first one which may look
 		 * a little bursty
 		 */
-		gettimeofday(&e, NULL);
-		usec = utime_since(&s, &e);
+		usec = utime_since(&s, &icd.time);
 
 		rate_throttle(td, usec, icd.bytes_done[td->ddir], td->ddir);
 
-		if (check_min_rate(td, &e)) {
+		if (check_min_rate(td, &icd.time)) {
 			if (exitall_on_terminate)
 				terminate_threads(td->groupid);
 			td_verror(td, ENOMEM);
 			break;
 		}
 
-		if (runtime_exceeded(td, &e))
+		if (runtime_exceeded(td, &icd.time))
 			break;
 
-		if (td->thinktime)
-			usec_sleep(td, td->thinktime);
+		if (td->thinktime) {
+			unsigned long long b;
+
+			b = td->io_blocks[0] + td->io_blocks[1];
+			if (!(td->thinktime_blocks % b))
+				usec_sleep(td, td->thinktime);
+		}
 	}
 
-	if (!ret) {
+	if (!td->error) {
 		if (td->cur_depth)
 			cleanup_pending_aio(td);
 
@@ -508,7 +512,12 @@ static int init_io_u(struct thread_data *td)
 		max_units = td->iodepth;
 
 	max_bs = max(td->max_bs[DDIR_READ], td->max_bs[DDIR_WRITE]);
-	td->orig_buffer_size = max_bs * max_units + MASK;
+	td->orig_buffer_size = max_bs * max_units;
+
+	if (td->mem_type == MEM_SHMHUGE || td->mem_type == MEM_MMAPHUGE)
+		td->orig_buffer_size = (td->orig_buffer_size + td->hugepage_size - 1) & ~(td->hugepage_size - 1);
+	else
+		td->orig_buffer_size += MASK;
 
 	if (allocate_io_mem(td))
 		return 1;
@@ -606,6 +615,7 @@ static void clear_io_state(struct thread_data *td)
  */
 static void *thread_main(void *data)
 {
+	unsigned long long runtime[2];
 	struct thread_data *td = data;
 
 	if (!td->use_thread)
@@ -656,15 +666,18 @@ static void *thread_main(void *data)
 
 	if (!td->create_serialize && setup_files(td))
 		goto err;
-
-	gettimeofday(&td->epoch, NULL);
+	if (open_files(td))
+		goto err;
 
 	if (td->exec_prerun)
 		system(td->exec_prerun);
 
+	fio_gettime(&td->epoch, NULL);
+	getrusage(RUSAGE_SELF, &td->ru_start);
+
+	runtime[0] = runtime[1] = 0;
 	while (td->loops--) {
-		getrusage(RUSAGE_SELF, &td->ru_start);
-		gettimeofday(&td->start, NULL);
+		fio_gettime(&td->start, NULL);
 		memcpy(&td->stat_sample_time, &td->start, sizeof(td->start));
 
 		if (td->ratemin)
@@ -678,11 +691,9 @@ static void *thread_main(void *data)
 		else
 			do_io(td);
 
-		td->runtime[td->ddir] += mtime_since_now(&td->start);
+		runtime[td->ddir] += utime_since_now(&td->start);
 		if (td_rw(td) && td->io_bytes[td->ddir ^ 1])
-			td->runtime[td->ddir ^ 1] = td->runtime[td->ddir];
-
-		update_rusage_stat(td);
+			runtime[td->ddir ^ 1] = runtime[td->ddir];
 
 		if (td->error || td->terminate)
 			break;
@@ -691,16 +702,21 @@ static void *thread_main(void *data)
 			continue;
 
 		clear_io_state(td);
-		gettimeofday(&td->start, NULL);
+		fio_gettime(&td->start, NULL);
 
 		do_verify(td);
 
-		td->runtime[DDIR_READ] += mtime_since_now(&td->start);
+		runtime[DDIR_READ] += utime_since_now(&td->start);
 
 		if (td->error || td->terminate)
 			break;
 	}
 
+	update_rusage_stat(td);
+	fio_gettime(&td->end_time, NULL);
+	td->runtime[0] = runtime[0] / 1000;
+	td->runtime[1] = runtime[1] / 1000;
+
 	if (td->bw_log)
 		finish_log(td, td->bw_log, "bw");
 	if (td->slat_log)
@@ -751,12 +767,12 @@ static void *fork_main(int shmid, int offset)
 static void reap_threads(int *nr_running, int *t_rate, int *m_rate)
 {
 	struct thread_data *td;
-	int i, cputhreads;
+	int i, cputhreads, pending;
 
 	/*
 	 * reap exited threads (TD_EXITED -> TD_REAPED)
 	 */
-	cputhreads = 0;
+	pending = cputhreads = 0;
 	for_each_td(td, i) {
 		/*
 		 * ->io_ops is NULL for a thread that has closed its
@@ -765,8 +781,12 @@ static void reap_threads(int *nr_running, int *t_rate, int *m_rate)
 		if (td->io_ops && td->io_ops->flags & FIO_CPUIO)
 			cputhreads++;
 
-		if (td->runstate != TD_EXITED)
+		if (td->runstate != TD_EXITED) {
+			if (td->runstate < TD_RUNNING)
+				pending++;
+
 			continue;
+		}
 
 		td_set_runstate(td, TD_REAPED);
 
@@ -783,7 +803,7 @@ static void reap_threads(int *nr_running, int *t_rate, int *m_rate)
 		(*t_rate) -= td->rate;
 	}
 
-	if (*nr_running == cputhreads)
+	if (*nr_running == cputhreads && !pending)
 		terminate_threads(TERMINATE_ALL);
 }
 
@@ -831,8 +851,6 @@ static void run_threads(void)
 		}
 	}
 
-	time_init();
-
 	while (todo) {
 		struct thread_data *map[MAX_JOBS];
 		struct timeval this_start;
@@ -892,7 +910,7 @@ static void run_threads(void)
 		 * Wait for the started threads to transition to
 		 * TD_INITIALIZED.
 		 */
-		gettimeofday(&this_start, NULL);
+		fio_gettime(&this_start, NULL);
 		left = this_jobs;
 		while (left) {
 			if (mtime_since_now(&this_start) > JOB_START_TIMEOUT)
