@@ -18,8 +18,15 @@
 #include "fio.h"
 #include "os.h"
 
+static LIST_HEAD(engine_list);
+
 static int check_engine_ops(struct ioengine_ops *ops)
 {
+	if (ops->version != FIO_IOOPS_VERSION) {
+		log_err("bad ioops version %d (want %d)\n", ops->version, FIO_IOOPS_VERSION);
+		return 1;
+	}
+
 	/*
 	 * cpu thread doesn't need to provide anything
 	 */
@@ -42,11 +49,74 @@ static int check_engine_ops(struct ioengine_ops *ops)
 	return 0;
 }
 
+void unregister_ioengine(struct ioengine_ops *ops)
+{
+	list_del(&ops->list);
+	INIT_LIST_HEAD(&ops->list);
+}
+
+int register_ioengine(struct ioengine_ops *ops)
+{
+	if (check_engine_ops(ops))
+		return 1;
+
+	INIT_LIST_HEAD(&ops->list);
+	list_add_tail(&ops->list, &engine_list);
+	return 0;
+}
+
+static struct ioengine_ops *find_ioengine(const char *name)
+{
+	struct ioengine_ops *ops;
+	struct list_head *entry;
+	char engine[16];
+
+	strncpy(engine, name, sizeof(engine) - 1);
+
+	if (!strncmp(engine, "linuxaio", 8) || !strncmp(engine, "aio", 3))
+		strcpy(engine, "libaio");
+
+	list_for_each(entry, &engine_list) {
+		ops = list_entry(entry, struct ioengine_ops, list);
+		if (!strcmp(engine, ops->name))
+			return ops;
+	}
+
+	return NULL;
+}
+
+static struct ioengine_ops *dlopen_ioengine(struct thread_data *td,
+					    const char *engine_lib)
+{
+	struct ioengine_ops *ops;
+	void *dlhandle;
+
+	dlerror();
+	dlhandle = dlopen(engine_lib, RTLD_LAZY);
+	if (!dlhandle) {
+		td_vmsg(td, -1, dlerror());
+		return NULL;
+	}
+
+	/*
+	 * Unlike the included modules, external engines should have a
+	 * non-static ioengine structure that we can reference.
+	 */
+	ops = dlsym(dlhandle, "ioengine");
+	if (!ops) {
+		td_vmsg(td, -1, dlerror());
+		dlclose(dlhandle);
+		return NULL;
+	}
+
+	ops->dlhandle = dlhandle;
+	return ops;
+}
+
 struct ioengine_ops *load_ioengine(struct thread_data *td, const char *name)
 {
-	char engine[16], engine_lib[256];
 	struct ioengine_ops *ops, *ret;
-	void *dlhandle;
+	char engine[16];
 
 	strncpy(engine, name, sizeof(engine) - 1);
 
@@ -56,39 +126,24 @@ struct ioengine_ops *load_ioengine(struct thread_data *td, const char *name)
 	if (!strncmp(engine, "linuxaio", 8) || !strncmp(engine, "aio", 3))
 		strcpy(engine, "libaio");
 
-	sprintf(engine_lib, "%s/lib/fio/fio-engine-%s.o", fio_inst_prefix, engine);
-	dlerror();
-	dlhandle = dlopen(engine_lib, RTLD_LAZY);
-	if (!dlhandle) {
-		td_vmsg(td, -1, dlerror());
-		return NULL;
-	}
+	ops = find_ioengine(engine);
+	if (!ops)
+		ops = dlopen_ioengine(td, name);
 
-	ops = dlsym(dlhandle, "ioengine");
 	if (!ops) {
-		td_vmsg(td, -1, dlerror());
-		dlclose(dlhandle);
-		return NULL;
-	}
-
-	if (ops->version != FIO_IOOPS_VERSION) {
-		log_err("bad ioops version %d (want %d)\n", ops->version, FIO_IOOPS_VERSION);
-		dlclose(dlhandle);
+		log_err("fio: engine %s not loadable\n", name);
 		return NULL;
 	}
 
 	/*
 	 * Check that the required methods are there.
 	 */
-	if (check_engine_ops(ops)) {
-		dlclose(dlhandle);
+	if (check_engine_ops(ops))
 		return NULL;
-	}
 
 	ret = malloc(sizeof(*ret));
 	memcpy(ret, ops, sizeof(*ret));
 	ret->data = NULL;
-	ret->dlhandle = dlhandle;
 
 	return ret;
 }
@@ -98,7 +153,9 @@ void close_ioengine(struct thread_data *td)
 	if (td->io_ops->cleanup)
 		td->io_ops->cleanup(td);
 
-	dlclose(td->io_ops->dlhandle);
+	if (td->io_ops->dlhandle)
+		dlclose(td->io_ops->dlhandle);
+
 	free(td->io_ops);
 	td->io_ops = NULL;
 }
@@ -119,7 +176,7 @@ int td_io_getevents(struct thread_data *td, int min, int max,
 
 int td_io_queue(struct thread_data *td, struct io_u *io_u)
 {
-	gettimeofday(&io_u->issue_time, NULL);
+	fio_gettime(&io_u->issue_time, NULL);
 
 	return td->io_ops->queue(td, io_u);
 }
