@@ -8,7 +8,6 @@
 #include "fio.h"
 #include "os.h"
 
-extern unsigned long long mlock_size;
 static void *pinned_mem;
 
 void fio_unpin_memory(void)
@@ -62,8 +61,13 @@ int allocate_io_mem(struct thread_data *td)
 {
 	if (td->mem_type == MEM_MALLOC)
 		td->orig_buffer = malloc(td->orig_buffer_size);
-	else if (td->mem_type == MEM_SHM) {
-		td->shm_id = shmget(IPC_PRIVATE, td->orig_buffer_size, IPC_CREAT | 0600);
+	else if (td->mem_type == MEM_SHM || td->mem_type == MEM_SHMHUGE) {
+		int flags = IPC_CREAT | SHM_R | SHM_W;
+
+		if (td->mem_type == MEM_SHMHUGE)
+			flags |= SHM_HUGETLB;
+
+		td->shm_id = shmget(IPC_PRIVATE, td->orig_buffer_size, flags);
 		if (td->shm_id < 0) {
 			td_verror(td, errno);
 			perror("shmget");
@@ -77,12 +81,33 @@ int allocate_io_mem(struct thread_data *td)
 			td->orig_buffer = NULL;
 			return 1;
 		}
-	} else if (td->mem_type == MEM_MMAP) {
-		td->orig_buffer = mmap(NULL, td->orig_buffer_size, PROT_READ | PROT_WRITE, MAP_PRIVATE | OS_MAP_ANON, 0, 0);
+	} else if (td->mem_type == MEM_MMAP || td->mem_type == MEM_MMAPHUGE) {
+		int flags = MAP_PRIVATE;
+
+		td->mmapfd = 0;
+
+		if (td->mmapfile) {
+			td->mmapfd = open(td->mmapfile, O_RDWR|O_CREAT, 0644);
+
+			if (td->mmapfd < 0) {
+				td_verror(td, errno);
+				perror("open mmap file");
+				td->orig_buffer = NULL;
+				return 1;
+			}
+		} else
+			flags |= OS_MAP_ANON;
+
+		td->orig_buffer = mmap(NULL, td->orig_buffer_size, PROT_READ | PROT_WRITE, flags, td->mmapfd, 0);
 		if (td->orig_buffer == MAP_FAILED) {
 			td_verror(td, errno);
 			perror("mmap");
 			td->orig_buffer = NULL;
+			if (td->mmapfd) {
+				close(td->mmapfd);
+				unlink(td->mmapfile);
+			}
+				
 			return 1;
 		}
 	}
@@ -94,15 +119,20 @@ void free_io_mem(struct thread_data *td)
 {
 	if (td->mem_type == MEM_MALLOC)
 		free(td->orig_buffer);
-	else if (td->mem_type == MEM_SHM) {
+	else if (td->mem_type == MEM_SHM || td->mem_type == MEM_SHMHUGE) {
 		struct shmid_ds sbuf;
 
 		shmdt(td->orig_buffer);
 		shmctl(td->shm_id, IPC_RMID, &sbuf);
-	} else if (td->mem_type == MEM_MMAP)
+	} else if (td->mem_type == MEM_MMAP || td->mem_type == MEM_MMAPHUGE) {
 		munmap(td->orig_buffer, td->orig_buffer_size);
-	else
-		log_err("Bad memory type %d\n", td->mem_type);
+		if (td->mmapfile) {
+			close(td->mmapfd);
+			unlink(td->mmapfile);
+			free(td->mmapfile);
+		}
+	} else
+		log_err("Bad memory type %u\n", td->mem_type);
 
 	td->orig_buffer = NULL;
 }
