@@ -60,7 +60,7 @@ static void update_io_tick_disk(struct disk_util *du)
 	dus->io_ticks += (__dus.io_ticks - ldus->io_ticks);
 	dus->time_in_queue += (__dus.time_in_queue - ldus->time_in_queue);
 
-	gettimeofday(&t, NULL);
+	fio_gettime(&t, NULL);
 	du->msec += mtime_since(&du->time, &t);
 	memcpy(&du->time, &t, sizeof(t));
 	memcpy(ldus, &__dus, sizeof(__dus));
@@ -94,15 +94,27 @@ static int disk_util_exists(dev_t dev)
 
 static void disk_util_add(dev_t dev, char *path)
 {
-	struct disk_util *du = malloc(sizeof(*du));
+	struct disk_util *du, *__du;
+	struct list_head *entry;
 
+	du = malloc(sizeof(*du));
 	memset(du, 0, sizeof(*du));
 	INIT_LIST_HEAD(&du->list);
 	sprintf(du->path, "%s/stat", path);
 	du->name = strdup(basename(path));
 	du->dev = dev;
 
-	gettimeofday(&du->time, NULL);
+	list_for_each(entry, &disk_list) {
+		__du = list_entry(entry, struct disk_util, list);
+
+		if (!strcmp(du->name, __du->name)) {
+			free(du->name);
+			free(du);
+			return;
+		}
+	}
+
+	fio_gettime(&du->time, NULL);
 	get_io_ticks(du, &du->last_dus);
 
 	list_add_tail(&du->list, &disk_list);
@@ -258,9 +270,6 @@ void disk_util_timer_arm(void)
 
 void update_rusage_stat(struct thread_data *td)
 {
-	if (!(td->runtime[0] + td->runtime[1]))
-		return;
-
 	getrusage(RUSAGE_SELF, &td->ru_end);
 
 	td->usr_time += mtime_since(&td->ru_start.ru_utime, &td->ru_end.ru_utime);
@@ -274,7 +283,7 @@ void update_rusage_stat(struct thread_data *td)
 static int calc_lat(struct io_stat *is, unsigned long *min, unsigned long *max,
 		    double *mean, double *dev)
 {
-	double n;
+	double n, o;
 
 	if (is->samples == 0)
 		return 0;
@@ -284,7 +293,16 @@ static int calc_lat(struct io_stat *is, unsigned long *min, unsigned long *max,
 
 	n = (double) is->samples;
 	*mean = (double) is->val / n;
-	*dev = sqrt(((double) is->val_sq - (*mean * *mean) / n) / (n - 1));
+	*dev = 0.01;
+
+	if (n <= 1.0)
+		return 1;
+
+	o = (double) is->val_sq - ((*mean * *mean) / n);
+	if (o < 0.0)
+		*dev = -1.0;
+	else
+		*dev = sqrt(o / (n - 1.0));
 
 	return 1;
 }
@@ -362,6 +380,7 @@ static void show_thread_status(struct thread_data *td,
 			       struct group_run_stats *rs)
 {
 	double usr_cpu, sys_cpu;
+	unsigned long runtime;
 
 	if (!(td->io_bytes[0] + td->io_bytes[1]) && !td->error)
 		return;
@@ -372,8 +391,9 @@ static void show_thread_status(struct thread_data *td,
 	if (td->io_bytes[td->ddir ^ 1])
 		show_ddir_status(td, rs, td->ddir ^ 1);
 
-	if (td->runtime[0] + td->runtime[1]) {
-		double runt = td->runtime[0] + td->runtime[1];
+	runtime = mtime_since(&td->epoch, &td->end_time);
+	if (runtime) {
+		double runt = (double) runtime;
 
 		usr_cpu = (double) td->usr_time * 100 / runt;
 		sys_cpu = (double) td->sys_time * 100 / runt;
@@ -430,7 +450,7 @@ static void show_thread_status_terse(struct thread_data *td,
 	show_ddir_status_terse(td, rs, 1);
 
 	if (td->runtime[0] + td->runtime[1]) {
-		double runt = td->runtime[0] + td->runtime[1];
+		double runt = (double) (td->runtime[0] + td->runtime[1]);
 
 		usr_cpu = (double) td->usr_time * 100 / runt;
 		sys_cpu = (double) td->sys_time * 100 / runt;
@@ -543,7 +563,7 @@ static inline void add_stat_sample(struct io_stat *is, unsigned long val)
 }
 
 static void add_log_sample(struct thread_data *td, struct io_log *iolog,
-			   unsigned long val, int ddir)
+			   unsigned long val, enum fio_ddir ddir)
 {
 	if (iolog->nr_samples == iolog->max_samples) {
 		int new_size = sizeof(struct io_sample) * iolog->max_samples*2;
@@ -558,7 +578,8 @@ static void add_log_sample(struct thread_data *td, struct io_log *iolog,
 	iolog->nr_samples++;
 }
 
-void add_clat_sample(struct thread_data *td, int ddir, unsigned long msec)
+void add_clat_sample(struct thread_data *td, enum fio_ddir ddir,
+		     unsigned long msec)
 {
 	add_stat_sample(&td->clat_stat[ddir], msec);
 
@@ -566,7 +587,8 @@ void add_clat_sample(struct thread_data *td, int ddir, unsigned long msec)
 		add_log_sample(td, td->clat_log, msec, ddir);
 }
 
-void add_slat_sample(struct thread_data *td, int ddir, unsigned long msec)
+void add_slat_sample(struct thread_data *td, enum fio_ddir ddir,
+		     unsigned long msec)
 {
 	add_stat_sample(&td->slat_stat[ddir], msec);
 
@@ -574,9 +596,10 @@ void add_slat_sample(struct thread_data *td, int ddir, unsigned long msec)
 		add_log_sample(td, td->slat_log, msec, ddir);
 }
 
-void add_bw_sample(struct thread_data *td, int ddir)
+void add_bw_sample(struct thread_data *td, enum fio_ddir ddir,
+		   struct timeval *t)
 {
-	unsigned long spent = mtime_since_now(&td->stat_sample_time[ddir]);
+	unsigned long spent = mtime_since(&td->stat_sample_time[ddir], t);
 	unsigned long rate;
 
 	if (spent < td->bw_avg_time)
@@ -588,7 +611,7 @@ void add_bw_sample(struct thread_data *td, int ddir)
 	if (td->bw_log)
 		add_log_sample(td, td->bw_log, rate, ddir);
 
-	gettimeofday(&td->stat_sample_time[ddir], NULL);
+	fio_gettime(&td->stat_sample_time[ddir], NULL);
 	td->stat_io_bytes[ddir] = td->this_io_bytes[ddir];
 }
 
